<?php

namespace Teknisa\EST\Modules\Ajuste\Services;

use Teknisa\EST\Modules\Ajuste\Est43000ManutencaoDoUltimoCustoFactory;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

    
class Est43000ManutencaoDoUltimoCustoService  {
    protected $factory;
    protected $dataSourceManager;
    
    public function __construct($factory, $dataSourceManager, $environment) {
        $this->factory = $factory;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment      = $environment;
    }
    
    public function saveManutencao($row) {
        $row['VRLIQCMPPRM1'] = 0;
        $row['VRLIQCMPPRM2'] = 0;
        $row['VRBRUCMPPRM1'] = 0;
        $row['VRBRUCMPPRM2']  = 0;
        $verificaIndice1 = $this->verificaIndice($row, 1);
        if(isset($verificaIndice1[0])) {
            $row['VRULTCMPLM1'] = $row['VRLIQCMPPROD'] * $verificaIndice1[0]['VRINDICE'];
            $row['VRULTCMPBM1'] = $row['VRBRUCMPPROD'] * $verificaIndice1[0]['VRINDICE'];
        }
        $verificaIndice2 = $this->verificaIndice($row, 2);
        if(isset($verificaIndice2[0])) {
            $row['VRULTCMPLM2'] = $row['VRLIQCMPPROD'] * $verificaIndice2[0]['VRINDICE'];
            $row['VRULTCMPBM2'] = $row['VRBRUCMPPROD'] * $verificaIndice2[0]['VRINDICE'];
        }
        if($row['__is_new']) {
            $row = $this->adicionaCampos($row);
        }
        $rowDataset = array();
        $rowDataset[0] = (array) $row;
        /*$dataSetLote = new DataSet('lote', $rowDataset);
        $this->dataSourceManager->persist($dataSetLote);*/
        $dataSet = new DataSet('custocmprod_produto_prodfili', $rowDataset);
        $this->dataSourceManager->persist($dataSet);
    }   
        
        
        
    public function buscaProdutosParaImportacao
    ($CDFILIAL, $DTREFERENCIA, $CDLOCALESTOQ = NULL, $CDALMOXARIFE = NULL, $P_CDPRODINI, $P_CDPRODFIN) {
        $params = array(
            ":CDFILIAL" => $CDFILIAL,
            ":CDALMOXARIFE" => $CDALMOXARIFE,
            ":CDLOCALESTOQ" => $CDLOCALESTOQ,
            ":DTREFERENCIA" =>$DTREFERENCIA,
            ":P_CDPRODINI" => $P_CDPRODINI,
            ":P_CDPRODFIN" => $P_CDPRODFIN
        );
        return $this->factory->buscaProdutosParaImportacao($params); 
    }
    
    public function realizaConsultaBanco($dataSourceName, $conditions) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet;
    }
    
    public function preparaRowsImportacao($rows, $dadosDestino) {
        if(!isset($dadosDestino['CDALMOXARIFEDEST'])) $dadosDestino['CDALMOXARIFEDEST'] = ' ';
        if(!isset($dadosDestino['CDLOCALESTOQDEST'])) $dadosDestino['CDLOCALESTOQDEST'] = ' ';
        $rowsReturn = array();
        foreach ($rows as $row) {
            
            $row['CDFILIAL']     = $dadosDestino['CDFILIALDEST'];
            $row['CDALMOXARIFE'] = $dadosDestino['CDALMOXARIFEDEST'];
            $row['CDLOCALESTOQ'] = $dadosDestino['CDLOCALESTOQDEST'];
            $row['DTREFERENCIA'] = $dadosDestino['DTREFERENCIADEST'];
            $row['CDFILLANCEST'] = $dadosDestino['CDFILIALDEST'];
            $params = array(
                ":CDFILIAL" => $dadosDestino['CDFILIALDEST'],
                ":CDALMOXARIFE" => $dadosDestino['CDALMOXARIFEDEST'],
                ":CDLOCALESTOQ" => $dadosDestino['CDLOCALESTOQDEST'],
                ":DTREFERENCIA" =>$dadosDestino['DTREFERENCIADEST'],
                ":NRLOTEESTQ" => $row['NRLOTEESTQ'],
                ":NRSUBLOTE" => $row['NRSUBLOTE'],
                ":CDPRODUTO"   => $row['CDPRODUTO']
            );
            $paramsLote = array(
                ":CDFILIAL" => $dadosDestino['CDFILIALDEST'],
                ":NRLOTEESTQ" => $row['NRLOTEESTQ'],
                ":NRSUBLOTE" => $row['NRSUBLOTE'],
                ":CDPRODUTO"   => $row['CDPRODUTO']
            );
            $this->factory->deletaCusto($params);
            $this->factory->insereLote($paramsLote);
            $rowAdd = $this->adicionaCampos($row);
            array_push($rowsReturn, $rowAdd);
        }
        return $rowsReturn;
    }
    
    public function insereProdutosImportados($row) {
        $dataSet = new DataSet('custocmprod_produto_prodfili', $row);
        $this->dataSourceManager->persist($dataSet);
    }
     
    public function atualizaUltimoCusto($row) {
        if(!isset($row['CDALMOXARIFE'])) {
            $row['CDALMOXARIFE'] = ' ';
            $row['TALMOX'] = 'T';
        }
        else if($row['CDALMOXARIFE'] == null) {
            $row['TALMOX'] = 'T';
        }
        else {
            $row['TALMOX'] = '';
        }
        if(!isset($row['CDLOCALESTOQ'])) {
            $row['CDLOCALESTOQ'] = ' ';
            $row['TLOCAL'] = 'T';
        }
        else if($row['CDLOCALESTOQ'] == null) {
            $row['TLOCAL'] = 'T';
        }
        else{
            $row['TLOCAL'] = '';
        }
        $row['DTINICIAL'] = $row['DTREFERENCIA']['start'];
        $row['DTFINAL'] = $row['DTREFERENCIA']['end'];
        return $this->factory->atualizaUltimoCusto($row['CDFILIAL'], $row['P_CDPRODINI'], $row['P_CDPRODFIN'], $row['CDALMOXARIFE'], $row['CDLOCALESTOQ'], $row['TALMOX'], $row['TLOCAL'], $row['DTINICIAL'], $row['DTFINAL']);
    } 
     
    protected function adicionaCampos($row) {
        $nrorg = $this->environment->getNrorgTrab();
		$operador = $this->environment->getCdOperador();
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$row['NRORG'] = $nrorg;
		$row['DTINCLUSAO'] = $currentTime;
		$row['DTULTATU'] = $currentTime;
		$row['CDOPERINCLUSAO'] = $operador;
		$row['CDOPERULTATU'] = $operador;
		$row['IDATIVO'] = 'S';
		$row['NRORGINCLUSAO'] = $nrorg;
		$row['NRORGULTATU'] = $nrorg;
	    $row['__is_new']  = true;
		return $row;
    }
    
    protected function verificaIndice($row, $tipo) {
        if($tipo === 1) {
            $params = array('P_DTREFIND' => $row['DTCUSTCMPPRO']);
            return $this->factory->verificaIndice1($params);
        }
        else if ($tipo === 2) {
            $params = array('P_DTREFIND' => $row['DTCUSTCMPPRO']);
            return $this->factory->verificaIndice2($params);
        }
    }
}
