<?php
namespace Teknisa\EST\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Estoque\Entities\Lanctoestoq;
use Teknisa\Model\Estoque\Entities\Ctrlmoviestq;
use Teknisa\Model\Estoque\Entities\Itlanctoest;
use Teknisa\EST\Services\ValidacoesEstoqueService;

class Est57000TransferenciaEstoqueService{

    protected $environment;
    protected $dataSourceManager;
    protected $newCode;
    protected $est57000transferenciaestoquefactory;
    protected $nrorg;

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager,NewCode $newCode,$est57000transferenciaestoquefactory, $validacoesEstoqueService){
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->nrorg = $environment->getNrOrg();
        $this->est57000transferenciaestoquefactory = $est57000transferenciaestoquefactory;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function deletar($dataSet){
        $this->dataSourceManager->delete($dataSet);
    }

    public function geraNovoCodigo($contador, $nrorg, $interval,$tamanho){
        return $this->newCode->getCodeByTableName($contador, $nrorg ,$interval,$tamanho);
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }

    public function buscaDataProcessamento($row){
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            $data = $this->est57000transferenciaestoquefactory->findDataProcessamento($nrorg, $row['CDFILIAL']);
            return $data[0];
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaProdutosRequisicao($transferencia){
        try {
            $produtos = $this->est57000transferenciaestoquefactory->buscaProdutosRequisicao($this->environment->getNrorgTrab(), $transferencia['CDFILIAL'], $transferencia['NRREQUESTO']);
            return $produtos;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteTransferencia($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //CASO O TRANSFERENCIA FOI FEITA A PARTIR DE UMA REQUISIÇÃO, ATUALIZA O STATUS DA REQUISICAO
            if($row['NRREQUESTO']){
                //VERIFICA A REQUISIÇÃO COMPLEMENTAR COM LANÇAMENTO DE ESTOQUE
                $result = $this->est57000transferenciaestoquefactory->verificaReqComp($row['CDFILIAL'], $row['CDFILIAL'], $row['NRLANESTSAID'], NULL);
                if(!$result) {
                    //DELETA REQUISIÇÃO COMPLEMENTAR SEM LANÇAMENTO DE ESTOQUE
                    $this->est57000transferenciaestoquefactory->deletarRequisicao($row['CDFILIAL'], $row['CDFILIAL'], $row['NRLANESTSAID']);
                }
                else {
                    //ATUALIZA STATUS DA REQUISIÇÃO COMPLEMENTAR E ATUALIZA ITENS DA REQUISIÇÃO COMPLENTAR COM LANÇAMENTO DE ESTOQUE 
                    $this->updateRequisicaoStatus($row['CDFILIAL'], $result['NRREQUESTO'], 'A');
                    $this->est57000transferenciaestoquefactory->updateItemRequisicao($row['CDFILIAL'], $result['NRREQUESTO']);
                }
                //ATUALIZA STATUS DA REQUISIÇÃO E DOS ITENS DA REQUISIÇÃO
                $this->updateRequisicaoStatus($row['CDFILIAL'], $row['NRREQUESTO'], 'A');
                $params = array(':IDSTITREQU'=> 'A',
                    ':CDFILILANC'=>NULL,
                    ':NRLANCESTQ'=>NULL,
                    ':NRSEQUITEM'=>NULL,
                    ':CDFILIAL'=>$row['CDFILIAL'],
                    ':NRREQUESTO'=>$row['NRREQUESTO'],
                    ':OLD_NRLANCESTQ'=>$row['NRLANESTSAID'],
                    ':CDPRODUTO'=>NULL,
                    ':NRSEQITEM'=>NULL,
                    ':NRORG'=>$row['NRORG']
                );
                $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
            }
            $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($row['CDFILIAL'], $row['NRLANESTSAID'], $row['NRORG']);
            $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($row['CDFILIAL'], $row['NRLANESTENTR'], $row['NRORG']));
            $this->est57000transferenciaestoquefactory->deleteItlanctoest($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRLANESTENTR'],$row['NRLANESTSAID']);
            $this->est57000transferenciaestoquefactory->deleteCtrlmoviestq($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRCTRLMOVEST']);
            $this->est57000transferenciaestoquefactory->deleteLanctoestoq($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRLANESTENTR'],$row['NRLANESTSAID']);
            $this->validacoesEstoqueService->atualizaEstoque($row['CDFILIAL'], $row['NRLANESTENTR']);
            $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
            $this->validacoesEstoqueService->atualizaEstoque($row['CDFILIAL'], $row['NRLANESTSAID']);
            $connection->commit();

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function deleteProduto($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $produto = $rows['produtos'];
            $transferencia = $rows['transferencia'];
            $itensTransferencia = $this->est57000transferenciaestoquefactory->buscaItensTransferencia($this->environment->getNrorgTrab(), $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);

            // NAO SE PODE EXCLUIR O PRODUTO SE ELE FOR O UNICO DA REQUISICAO
            if(count($itensTransferencia) > 1 ){
                $produtosaida = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTSAID'],
                    )
                );
                $produtoentrada = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTENTR'],

                    )
                );
              
                
                //CASO O TRANSFERENCIA FOI FEITA A PARTIR DE UMA REQUISIÇÃO, ATUALIZA O STATUS DA REQUISICAO
                if($transferencia['NRREQUESTO']){
                    $prodDaRequisicao = false;
                    //VERIFICA SE O ITEM DA REQUISIÇÃO COMPLEMENTAR TEM LANÇAMENTO DE ESTOQUE
                    $produtosRequisicao = $this->buscaProdutosRequisicao($transferencia);
                    foreach ($produtosRequisicao as $produtoRequisicao) {
                        if($produto['CDPRODUTO'] == $produtoRequisicao['CDPRODUTO']) {
                            $prodDaRequisicao = true;
                        }
                    }
                    if($prodDaRequisicao == true) {
                        $result = $this->est57000transferenciaestoquefactory->verificaReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                        if(!$result) {
                            //DELETA O ITEM DA REQUISICAO COMPLEMENTAR SEM LANÇAMENTO DE ESTOQUE
                            $ultimoItem = !$this->est57000transferenciaestoquefactory->verificaUltimoItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                            if(!$ultimoItem) {
                                $this->est57000transferenciaestoquefactory->deletarRequisicao($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);
                            }
                            else {
                                $this->est57000transferenciaestoquefactory->deleteItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQUITEM']);
                            }
                        }
                        else {
                            //ATUALIZA STATUS DO ITEM DA REQUISIÇÃO COMPLEMENTAR E ATUALIZA ITENS DA REQUISIÇÃO COMPLENTAR COM LANÇAMENTO DE ESTOQUE 
                            $this->est57000transferenciaestoquefactory->updateItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                        }
                        //ATUALIZA STATUS DO ITEM DA REQUISIÇÃO
                        $this->updateRequisicaoStatus($transferencia['CDFILIAL'], $transferencia['NRREQUESTO'], 'A');
                        $params = array(':IDSTITREQU'=> 'A',
                            ':CDFILILANC'=>NULL,
                            ':NRLANCESTQ'=>NULL,
                            ':NRSEQUITEM'=>NULL,
                            ':CDFILIAL'=>$transferencia['CDFILIAL'],
                            ':NRREQUESTO'=>$transferencia['NRREQUESTO'],
                            ':OLD_NRLANCESTQ'=>$transferencia['NRLANESTSAID'],
                            ':CDPRODUTO'=>$produto['CDPRODUTO'], 
                            ':NRSEQITEM' =>NULL,
                            ':NRORG'=>$transferencia['NRORG']
                        );
                        $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
                    }
                }

            // Atualiza o estoque
            $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRORG']);
            $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($transferencia['CDFILIAL'], $transferencia['NRLANESTENTR'], $transferencia['NRORG']));
            $this->entityManager->remove($produtosaida);
            $this->entityManager->flush();
            $this->entityManager->remove($produtoentrada);
            $this->entityManager->flush();
            $this->validacoesEstoqueService->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTENTR']);
            $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
            $this->validacoesEstoqueService->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);
            $connection->commit();
            }else{
                throw new \Exception("Não foi possível exlcuir. Existe somente este produto cadastrado para a transferência.", 1);
            }
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function editProdutoTransferencia($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $produto = $rows['produtos'];
            $transferencia = $rows['transferencia'];
            $nrorg       = $this->environment->getNrorgTrab();
            $operador    = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
            $qttotlancto =  floatval ( $produto['QTTOTLANCTO']) ;
            $qtlanctoest =  floatval ( $produto['QTTOTLANCTO']  * $produto['VRFATOCONV'] );
            $vrtotlancto=   $produto['VRUNILANCTO'] * floatval ( $produto['QTTOTLANCTO'] );

            $produtosaida = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                array(  'nrorg' => $produto['NRORG'],
                        'cdfilial' => $produto['CDFILIAL'],
                        'nrsequitem' => $produto['NRSEQUITEM'],
                        'nrlancestq' => $transferencia['NRLANESTSAID'],
                )
            );
            $produtosaida->setQttotlancto($qttotlancto);
            $produtosaida->setQtlanctoest($qtlanctoest);
            $produtosaida->setVrtotlancto($vrtotlancto);
            $produtosaida->setNrorgultatu($nrorg);
            $produtosaida->setDtultatu($currentTime);
            $produtosaida->setCdoperultatu($operador);

            $produtoentrada = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                array(  'nrorg' => $produto['NRORG'],
                        'cdfilial' => $produto['CDFILIAL'],
                        'nrsequitem' => $produto['NRSEQUITEM'],
                        'nrlancestq' => $transferencia['NRLANESTENTR'],

                )
            );
            $produtoentrada->setQttotlancto($qttotlancto);
            $produtoentrada->setQtlanctoest($qtlanctoest);
            $produtoentrada->setVrtotlancto($vrtotlancto);
            $produtoentrada->setNrorgultatu($nrorg);
            $produtoentrada->setDtultatu($currentTime);
            $produtoentrada->setCdoperultatu($operador);
        
            $this->entityManager->persist($produtosaida);
            $this->entityManager->flush();

            $this->entityManager->persist($produtoentrada);
            $this->entityManager->flush();
            $this->atualizaEstoque($produto['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRLANESTENTR'], $produto['NRORG']);
            
            $array = [
                "qttotlancto" => $qttotlancto,
                "qtlanctoest" => $qtlanctoest,
                "vrtotlancto" => $vrtotlancto
            ];
            $dataSet = new DataSet('produto', $array);
            $connection->commit();
            return $dataSet;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function salvarNovaTransferencia($transferencia, $produto){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $transferencia['NRORG'] = $this->environment->getNrorgTrab();

            $nrctrlmovest = $this->geraNovoCodigo('NRTRANSESTQ'.$transferencia['CDFILIAL'],$this->environment->getNrOrgTrab(),1,10);
            
            //realiza o lançamento de estoque  referente a saída e entrada  ------ 4-Saida 2 -Entrada
            $nrlancamentsaida    = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,4);
            $nrlancamentoentrada = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,2);

            $dataSet = $this->saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada);

            //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,[$produto],4,$nrlancamentsaida);
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,[$produto],2,$nrlancamentoentrada);
            
            $this->atualizaEstoque($transferencia['CDFILIAL'], $nrlancamentsaida, $nrlancamentoentrada, $transferencia['NRORG']);
            $connection->commit();
            return $dataSet;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function salvarNovaTransferenciaComRequisicao($transferencia, $produtos){
        try {
            $nrrequesto = $transferencia['NRREQUESTO'];
            $transferencia['NRORG'] = $this->environment->getNrOrgTrab();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $produtos = $this->preparaProdutos($transferencia, $produtos);
            $nrctrlmovest = $this->geraNovoCodigo('NRTRANSESTQ'.$transferencia['CDFILIAL'],$this->environment->getNrOrgTrab(),1,10);
            $transferencia['CDFILIMOVI'] = $transferencia['CDFILIAL'];
            //realiza o lançamento de estoque  referente a saída e entrada  ------ 4-Saida 2 -Entrada
            $nrlancamentsaida    = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,4);
            $nrlancamentoentrada = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,2);
            $transferencia['NRLANESTSAID'] = $nrlancamentsaida;
            $transferencia['NRLANESTENTR'] = $nrlancamentoentrada;
            
            // valida e atualiza a requisição
            

            $dataSet = $this->saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada);
            //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,4,$nrlancamentsaida, $nrsequitemrequest = true);
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,2,$nrlancamentoentrada);
            $this->validateRequisicao($transferencia, $produtos);
            // gera requisição complementar conforme parametrização
            $this->validateReqComp($transferencia, $produtos);
            $this->atualizaEstoque($transferencia['CDFILIAL'], $nrlancamentsaida, $nrlancamentoentrada, $transferencia['NRORG']);
            
            $connection->commit();
            return $dataSet;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function preparaProdutos($transferencia, $produtos) {
        $nrsequitem = 1;
        $contador = 0;
        foreach ($produtos as $produto) {
            $produtos[$contador]['NRSEQUITEM'] = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
            $produtos[$contador]['NRSEQITEMREQC'] = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
            $nrsequitem ++;
            $contador++;
        }
        return $produtos;
    }

    public function salvarNovoProduto($transferencia, $produto){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            
            //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
            $this->saveTabelaItemLancamentoEstoque($transferencia['NRCTRLMOVEST'], $transferencia, [$produto], 4, $transferencia['NRLANESTSAID']);
            $this->saveTabelaItemLancamentoEstoque($transferencia['NRCTRLMOVEST'], $transferencia, [$produto], 2, $transferencia['NRLANESTENTR']);
            
            $this->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRLANESTENTR'], $transferencia['NRORG']);
            

            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,$identradaSaida){
        try {
            $nrorg = $this->environment->getNrorgTrab();
            $operador = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
            $nrlancestq = $this->geraNovoCodigo('LANCTOEST'.$transferencia['CDFILIAL'],$nrorg,1,10);

            $lanctoestoq = new Lanctoestoq();
            $lanctoestoq->setNrorg($nrorg);
            $lanctoestoq->setDtinclusao($currentTime);
            $lanctoestoq->setCdoperinclusao($operador);
            $lanctoestoq->setNrorginclusao($nrorg);
            $lanctoestoq->setNrorgultatu($nrorg);
            $lanctoestoq->setDtultatu($currentTime);
            $lanctoestoq->setCdoperultatu($operador);
            $lanctoestoq->setIdativo("S");

            $lanctoestoq->setCdfilial($transferencia['CDFILIAL']);
            $lanctoestoq->setNrlancestq($nrlancestq);
            $lanctoestoq->setIdtplancto($identradaSaida);
            $lanctoestoq->setDtlancestq ( DateUtil::getDataDeString($transferencia['DTMOVIESTQ']) );
            $lanctoestoq->setDslancestq('Transferência de Estoque '.$nrctrlmovest);
            $lanctoestoq->setCdfilimovi($transferencia['CDFILIMOVI']);
            $lanctoestoq->setCdoperador($operador);

            $this->entityManager->persist($lanctoestoq);
            $this->entityManager->flush();
            return $nrlancestq;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada){
        try {
            $nrorg = $this->environment->getNrorgTrab();
            $operador = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
           
            $cdlcestentr = $transferencia['CDLCESTENTR'];
            $cdlcestsaid = $transferencia['CDLCESTSAID'];
            if(!$cdlcestentr){
                $cdlcestentr = ' ';
            }
            if(!$cdlcestsaid){
                $cdlcestsaid = ' ';
            }
            
            $ctrlmoviestq = new Ctrlmoviestq();
            $ctrlmoviestq->setNrorg($nrorg);
            $ctrlmoviestq->setDtinclusao($currentTime);
            $ctrlmoviestq->setCdoperinclusao($operador);
            $ctrlmoviestq->setNrorginclusao($nrorg);
            $ctrlmoviestq->setNrorgultatu($nrorg);
            $ctrlmoviestq->setDtultatu($currentTime);
            $ctrlmoviestq->setCdoperultatu($operador);
            $ctrlmoviestq->setIdativo("S");

            $ctrlmoviestq->setNrrequesto( $transferencia['NRREQUESTO'] );
            $ctrlmoviestq->setCdfilial( $transferencia['CDFILIMOVI'] );
            $ctrlmoviestq->setNrctrlmovest($nrctrlmovest);
            $ctrlmoviestq->setCdfilialentr($transferencia['CDFILIAL']);
            $ctrlmoviestq->setNrlanestentr($nrlancamentoentrada);
            $ctrlmoviestq->setCdfilialsaid($transferencia['CDFILIAL']);
            $ctrlmoviestq->setNrlanestsaid($nrlancamentsaida);
            $ctrlmoviestq->setCdalmoxentr($transferencia['CDALMOXENTR']);
            $ctrlmoviestq->setCdalmoxsaid($transferencia['CDALMOXSAID']);
            $ctrlmoviestq->setCdlcestentr($cdlcestentr);
            $ctrlmoviestq->setCdlcestsaid($cdlcestsaid);
            $ctrlmoviestq->setDtmoviestq( DateUtil::getDataDeString($transferencia['DTMOVIESTQ']) );

            $this->entityManager->persist($ctrlmoviestq);
            $this->entityManager->flush();
            $array = [
                "nrorg" => $nrorg,
                "nrlanestsaid" => $nrlancamentsaida,
                "nrlanestentr" => $nrlancamentoentrada,
                "nrctrlmovest" => $nrctrlmovest
            ];
            $dataSet = new DataSet('transferencia', $array);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,$identradaSaida,$nrlancamentoestoque, $saveNRSEQITEMREQUEST = false){
        try {
            $nrorg       = $this->environment->getNrorgTrab();
            $operador    = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $arrayItens  = [];
            $nrtipocusto = $this->est57000transferenciaestoquefactory->buscaTipoCustoFilial($nrorg,$transferencia['CDFILIAL']);

            $nrtipocusto = $nrtipocusto[0]['CDTIPCUSTEST'];
            foreach ($produtos as &$prod) {
                $cdalmoxarife = '';
                $cdlocalestoq = '';

                //item de saida
                if($identradaSaida ==4){
                    $cdalmoxarife = $transferencia['CDALMOXSAID'];
                    $cdlocalestoq = $transferencia['CDLCESTSAID'];
                    if(!$cdlocalestoq){
                        $cdlocalestoq =' ';
                    }
                }
                //item de entrada
                else{
                    $cdalmoxarife = $transferencia['CDALMOXENTR'];
                    $cdlocalestoq = $transferencia['CDLCESTENTR'];
                    if(!$cdlocalestoq){
                        $cdlocalestoq =' ';
                    }
                }

                if(!$prod['NRLOTEESTQ']){
                    $prod['NRLOTEESTQ']=' ';
                }

                if(!$prod['NRSUBLOTE']){
                    $prod['NRSUBLOTE']=' ';
                }

                $cdalmoxarifeCusto = $transferencia['CDALMOXSAID'];
                if(!$cdalmoxarifeCusto){
                    $cdalmoxarifeCusto =' ';
                }
                $cdlocalestoqCusto = $transferencia['CDLCESTSAID'];
                if(!$cdlocalestoqCusto){
                    $cdlocalestoqCusto =' ';
                }
                $vlrunitario  =  $this->est57000transferenciaestoquefactory->gerarValorUnitario($transferencia['CDFILIAL'], $prod['CDPRODUTO'], $nrtipocusto, $transferencia['DTMOVIESTQ'], $this->environment->getCurrentTime()->format("d/m/Y"), $cdalmoxarifeCusto, $cdlocalestoqCusto, $prod['NRLOTEESTQ'], $prod['NRSUBLOTE'])[0]['VRCUSTO'];
                
                $nrsequitem   = $this->est57000transferenciaestoquefactory->geraNrsequitem($nrorg,$transferencia['CDFILIAL'],$nrlancamentoestoque);

                if(isset($prod['NRREQUESTO'])){
                    if($prod['NRREQUESTO'] && $nrsequitem[0]['NRSEQUITEM'] === '0001') {
                        $nrsequitem = $prod['NRSEQUITEM'];
                    }
                    else {
                        $nrsequitem   = $nrsequitem[0]['NRSEQUITEM'];
                    }
                }
                else {
                    $nrsequitem   = $nrsequitem[0]['NRSEQUITEM'];
                }
                $row = new Row();
                $nrsequitemrequest = null;
                if($saveNRSEQITEMREQUEST) {
                    if(isset($prod['NRSEQITEM'])) $nrsequitemrequest = $prod['NRSEQITEM'];
                }

                $row->set('__is_new',true);
                $row->set('NRORG',$nrorg);
                $row->set('NRORGULTATU',$nrorg);
                $row->set('NRORGINCLUSAO',$nrorg);
                $row->set('DTULTATU',$currentTime);
                $row->set('DTINCLUSAO',$currentTime);
                $row->set('CDOPERINCLUSAO',$operador);
                $row->set('CDOPERULTATU',$operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$operador);
                $row->set('CDFILIAL',     $transferencia['CDFILIAL'] );
                $row->set('NRLANCESTQ',   $nrlancamentoestoque);
                $row->set('NRSEQUITEM',   $nrsequitem);
                $row->set('CDPRODUTO',    $prod['CDPRODUTO']);
                $row->set('CDALMOXARIFE', $cdalmoxarife);
                $row->set('CDLOCALESTOQ', $cdlocalestoq);
                $row->set('DSLANCESTIT',  'Transferência de Estoque '.$transferencia['NRCTRLMOVEST']);
                $row->set('NRLOTEESTQ',   $prod['NRLOTEESTQ']);
                $row->set('NRSUBLOTE',    $prod['NRSUBLOTE']);
                $row->set('CDFILIMOVI',   $transferencia['CDFILIMOVI']);
                $row->set('CDPRODMOVI',   $prod['CDPRODESTO']);
                $row->set('IDORDEMCLC',   $identradaSaida);
                $row->set('DTLANCMOVI' ,  $transferencia['DTMOVIESTQ']);
                $row->set('IDEVOLESTOQ',  'S');
                $row->set('IDTIPOMOVI',   $identradaSaida);
                $row->set('QTTOTLANCTO',  floatval ($prod['QTTOTLANCTO']));
                $row->set('QTLANCTOEST',  floatval ($prod['QTTOTLANCTO']) * ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRUNILANCTO' , $vlrunitario);
                $row->set('VRTOTLANCTO' , $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQDIA' ,  $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQBRUT' , ($vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV'])) * floatval($prod['QTTOTLANCTO']));
                $row->set('VRLANCTOEST' , $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRLANCTOBRUT', $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('NRLANCORIGRP',  null);
                $row->set('NRSEQITEMREQEST',  $nrsequitemrequest);
                $row->set('QTDEVOLESTOQ',  null);
                $arrayItens [] = $row;
            }
            $dataSet = new DataSet('itlanctoest_produto_itrequesto', $arrayItens);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getAlmoxRequisicao($nrrequesto) {
        $rows = $this->est57000transferenciaestoquefactory->getAlmoxRequisicao($nrrequesto);
        $dataSet = new DataSet('getAlmoxRequisicao', $rows);
        return $dataSet;
    }
    
    public function validateReqComp($lancamento, $itens) {
        if($this->shouldCreateReqComp($itens)) {
            $reqComp = $this->getRequisicaoComplementarAberta($lancamento);

            $this->insertItensForReqComp($reqComp, $lancamento, $itens);
        }
    }

    public function insertItensForReqComp($NRREQUESTO, $lancamento, $itens) {
        $params = [
            "NRREQUESTO" => $NRREQUESTO,
            "CDFILIAL" => $lancamento['CDFILIMOVI'],
            "QTDEVOREQU" => 0,
            "IDSTITREQU" => 'A',
            "CDFILILANC" => null,
            "NRLANCESTQ" => null,
            "NRSEQUITEM" => null,
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANESTSAID']
        ];
        $nrorg       = $this->environment->getNrorgTrab();
        $operador    = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        foreach ($itens as $item) {
            if(array_key_exists('createReqComp', $item) && $item['createReqComp']) {
                $params['NRSEQITREQC'] = $item['NRSEQUITEM'];
                $params['CDPRODUTO'] = $item['CDPRODUTO'];
                $params['NRLOTEESTQ'] = $item['NRLOTEESTQ'];
                $params['NRSUBLOTE'] = $item['NRSUBLOTE'];
                $params['QTREQUESTO'] = $item['QTLANCREQCOMP'];
                $params['NRORG']    = $nrorg;
                $params['NRORGULTATU'   ]    = $nrorg;
                $params['NRORGINCLUSAO']    = $nrorg;
                $params['DTULTATU']    = $currentTime;
                $params['DTINCLUSAO']    = $currentTime;
                $params['CDOPERINCLUSAO']    = $operador;
                $params['CDOPERULTATU']    = $operador;
                $params['IDATIVO'   ]    = 'S';
                $params['CDOPERADOR']    = $operador;



                $newCodeParams = implode('', [$lancamento['CDFILIAL'],
                                  $NRREQUESTO,
                                  $item['CDPRODUTO']]);

                $NRSEQITEM = $this->geraNewSequencial('SEQITREQESTO', $newCodeParams, 4);

                $params['NRSEQITEM'] = $NRSEQITEM;

                $this->est57000transferenciaestoquefactory->insertItrequestoComplementar($params);
            }
        }
    }

    public function shouldCreateReqComp($itens) {
        foreach ($itens as $item) {
            if(array_key_exists('createReqComp', $item) && $item['createReqComp'])
                return true;
        }

        return false;
    }

    public function getRequisicaoComplementarAberta($lancamento) {
        $params = [
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANESTSAID']
        ];
        $reqComp = $this->est57000transferenciaestoquefactory->getRequisicaoComplementarAberta($params);

        if(empty($reqComp)) {
            $NRREQUCOMPESTO = $this->geraNewSequencial('NRREQUESTO');

            $params = [
                "CDFILIMOVI" => $lancamento['CDFILIMOVI'],
                "NRREQUESTO" => $lancamento['NRREQUESTO']
            ];
            $reqCompParameters = $this->est57000transferenciaestoquefactory->getRequisicaoComplementarParameters($params);
            $params = [
                "CDFILIAL" => $lancamento['CDFILIAL'],
                "NRREQUESTO" => $NRREQUCOMPESTO,
                "DTREQUESTO" => date("m/d/Y", strtotime($lancamento['DTMOVIESTQ'])),
                "DTPREVREQU" => date("m/d/Y", strtotime($lancamento['DTMOVIESTQ'])),
                "DSREQUESTO" => 'Requisição Complementar - Req. Orig.: ' . $lancamento['NRREQUESTO'] . ' Baixa Orig.:  ' . $lancamento['NRLANESTSAID'],
                "CDSETOR" => $reqCompParameters['CDSETOR'],
                "CDCENTCUST" => $reqCompParameters['CDCENTCUST'],
                "CDOPERREQU" => $this->environment->getCdOperador(),
                "CDALMOXSAID" => $reqCompParameters['CDALMOXSAID'],
                "CDLCESTSAID" => $reqCompParameters['CDLCESTSAID'],
                "CDALMOXENTR" => $reqCompParameters['CDALMOXENTR'],
                "CDLCESTENTR" => $reqCompParameters['CDLCESTENTR'],
                "CDFILLANCREQC" => $lancamento['CDFILIAL'],
                "NRLANCREQC" => $lancamento['NRLANESTSAID']
            ];
            $this->est57000transferenciaestoquefactory->insertRequisicaoComplementar($params);

            return $NRREQUCOMPESTO;
        }

        return $reqComp['NRREQUESTO'];
    }

    /**
     * Essa função é chamda quando está se inserindo uma retirada com uma requisição.
     * Ela tem a finalidade de desabilitar a requisição que foi passada, para que não
     * se possa mais inserir retiradas com aquela requisição que foi passada.
     *
     * @param $lancamento | row com os dados do lançamento
     * @param $itensLancesto | array com rows com os dados de cada item do lançamento
     */
    public function validateRequisicao($lancamento, $itensLancesto) {
        $this->updateItemStatus($lancamento, $itensLancesto);
        
        $newReqStatus = $this->getProperRequisicaoStatus($lancamento);

        $this->updateRequisicaoStatus($lancamento["CDFILIAL"], $lancamento["NRREQUESTO"], $newReqStatus);
    }

    //atualiza os status dos itens de uma requisição na base
    public function updateItemStatus($lancamento, $itensLancesto) {
        $itensRequesto = [];
        foreach($itensLancesto as $itemLancesto) {
            if($itemLancesto['NRREQUESTO'] && $itemLancesto['NRSEQITEM']) {
                $itensRequesto[] = $itemLancesto;
            }
        }

        if($itensRequesto) {
            $this->encerrarItensRequesto($lancamento, $itensRequesto);
        }
    }
    
    public function updateRequisicaoStatus($CDFILIAL, $NRREQUESTO, $IDSTATUSREQ) {
        $IDREQUESTO = ($IDSTATUSREQ == 'A')? 'P' : 'C';
        $params = [
            "IDREQUESTO" => $IDREQUESTO,
            "IDSTATUSREQ" => $IDSTATUSREQ,
            "CDFILIAL" => $CDFILIAL,
            "NRREQUESTO" => $NRREQUESTO
        ];

        $this->est57000transferenciaestoquefactory->updateRequestoStatus($params);
    }

    //lança os CDPRODUTOs pra factory pra atualizar os status dos itens para "E"
    public function encerrarItensRequesto($lancamento, $itensRequesto) {
        $params = [
            "IDSTITREQU" => "E",
            "NRLANCESTQ" => $lancamento["NRLANESTSAID"],
            "NRORG" => $this->nrorg,
            "OLD_NRLANCESTQ" => null
        ];


        foreach ($itensRequesto as $item) {
            $params["CDFILIAL"]   = $item["CDFILIAL"];
            $params["CDFILILANC"] = $item["CDFILIAL"];
            $params["CDPRODUTO"]  = $item["CDPRODUTO"];
            $params["NRSEQITEM"]  = $item["NRSEQITEM"];
            $params["NRSEQUITEM"] = $item["NRSEQUITEM"];
            $params["NRREQUESTO"] = $item["NRREQUESTO"];
            $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
        }
    }
    
    public function getProperRequisicaoStatus($lancamento) {
        $params = [
            "CDFILIAL" => $lancamento['CDFILIAL'],
            "NRREQUESTO" => $lancamento['NRREQUESTO'],
            "NRORG" => $this->nrorg,
            "IDSTITREQU" => null
        ];

        $itensRequesto = $this->est57000transferenciaestoquefactory->getItensByRequisicao($params);

        $statusRequesto = $itensRequesto[0]["IDSTITREQU"];
        foreach ($itensRequesto as $itemRequesto) {
            if($itemRequesto["IDSTITREQU"]!= $statusRequesto) {
                $statusRequesto = "P";
            }
        }

        return $statusRequesto;
    }
    
    public function geraNewSequencial($table, $param = '', $colSize = 10) {
        return $this->newCode->getCodeByTableName($table.$param, $this->nrorg, 1, $colSize);
    }
    
    protected function atualizaEstoque($CDFILIAL, $NRLANESTSAID, $NRLANESTENTR, $NRORG) {
        $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($CDFILIAL, $NRLANESTSAID, $NRORG);
        $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
        $this->validacoesEstoqueService->atualizaEstoque($CDFILIAL, $NRLANESTSAID);
        $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($CDFILIAL, $NRLANESTENTR, $NRORG));
        $this->validacoesEstoqueService->atualizaEstoque($CDFILIAL, $NRLANESTENTR);
    }
} 